import { register } from "@antv/x6-react-shape";
import { EventArgs, Graph, Node, Path } from "@antv/x6";
import { topicData } from "@/config/data";
import { useSizeHook, useShapeProps } from "@/hooks";
import CustomInput from "../CustomInput";
import { useEffect, useMemo, useRef, useState } from "react";
import { PlusOutlined } from "@ant-design/icons";
import { TopicType } from "@/enum";
import { addTopic } from "@/pages/mindmap/mindMap";

import Link from "./Link";
import ExtraModule from "./ExtraModule";
import CustomTag from "@/components/CustomTag";
import { TopicItem } from "@/types";
import { selectTopic } from "@/utils/mindmapHander";
const component = ({ node, graph }: { node: Node; graph: Graph }) => {
  const {
    fill,
    stroke,
    opacity,
    label,
    text,
    borderSize,
    setMindProjectInfo,
    icons,
    tags,
    extraModules,
    remark,
    herf,
    children,
    type,
    collapsed,
  } = node.getData();
  const { size, ref } = useSizeHook();
  const { fillContent, strokeColor, strokeWidth, strokeDasharray } =
    useShapeProps(fill, size, stroke);
  const [selected, setSelected] = useState(false);
  const handleSelect = (_args: EventArgs["node:selected"]) => {
    const cells = graph.getSelectedCells();
    setSelected(!!cells.find((item) => item.id === node.id));
  };
  const [showCollapsePoint, setShowCollapsePoint] = useState(collapsed);
  const extraModuleRef = useRef<HTMLDivElement>(null);
  const titleRef = useRef<HTMLDivElement>(null);
  const tagRef = useRef<HTMLDivElement>(null);
  const remarkRef = useRef<HTMLDivElement>(null);

  useEffect(() => {
    // graph.createTransformWidget(node);
    // graph.select(node);
    graph.on("node:selected", handleSelect);
    graph.on("node:unselected", handleSelect);
    return () => {
      graph.off("node:selected", handleSelect);
      graph.off("node:unselected", handleSelect);
    };
  }, []);

  useEffect(() => {
    // 动态计算出所需的宽高
    let width = extraModules?.width || 0;
  }, [extraModules, label, icons, tags, remarkRef]);

  const childrenCount = useMemo(() => {
    let count = 0;
    const traverse = (topics: TopicItem[]) => {
      topics.forEach((item) => {
        count++;
        if (item.children) {
          traverse(item.children);
        }
      });
    };

    traverse(children);
    return count;
  }, [children]);

  const handleAddBranch = () => {
    const data = node.getData();
    let topic;
    if (data.type === TopicType.main) {
      topic = addTopic(TopicType.branch, setMindProjectInfo, node);
    } else {
      topic = addTopic(TopicType.sub, setMindProjectInfo, node);
    }
    
    selectTopic(graph, topic);
  };

  const handleToggleCollapse = () => {
    node.setData({
      collapsed: !collapsed,
    });
  };

  return (
    <>
      {selected && (
        <div
          style={{
            width: `calc(100% + 4px)`,
            height: `calc(100% + 4px)`,
            border: "1.5px solid #239edd",
            position: "absolute",
            top: -2,
            left: -2,
          }}
        />
      )}
      <div
        className="relative text-0 w-full h-full px-4px"
        ref={ref}
        style={{
          opacity: opacity / 100,
          border: `solid ${strokeWidth}px ${strokeColor}`,
          background: fillContent,
          borderRadius: borderSize,
        }}
        onMouseOver={() => !collapsed && setShowCollapsePoint(true)}
        onMouseLeave={() => !collapsed && setShowCollapsePoint(false)}
      >
        <div className="w-full h-full flex flex-col">
          {extraModules && (
            <div ref={extraModuleRef}>
              <ExtraModule node={node} extraModules={extraModules} />
            </div>
          )}

          <div className="flex-1 flex">
            <div
              className="flex-1 flex flex-col justify-center"
              // style={{ height: getSize.contentHeight }}
            >
              <div
                className="flex justify-start items-center text-20px"
                ref={titleRef}
              >
                {icons?.map((icon: string) => {
                  return (
                    <svg key={icon} className="icon mr-6px" aria-hidden="true">
                      <use xlinkHref={`#${icon}`}></use>
                    </svg>
                  );
                })}
                <CustomInput
                  value={label}
                  node={node}
                  styles={{
                    ...text,
                    position: "relative",
                  }}
                  txtStyle={{
                    position: "relative",
                    flex: 1,
                  }}
                />
              </div>

              <div className="flex" ref={tagRef}>
                {tags?.map((item: { name: string; color: string }) => {
                  return (
                    <CustomTag
                      className="text-14px"
                      key={item.name}
                      title={item.name}
                      color={item.color}
                    >
                      {item.name}
                    </CustomTag>
                  );
                })}
              </div>
            </div>

            <div ref={remarkRef}>
              {herf && <Link link={herf} />}
              {remark && <i className="iconfont icon-pinglun1" />}
            </div>
          </div>
        </div>

        {/* 添加主题按钮 */}
        {selected && !children?.length && (
          <div
            className={`
              absolute 
              w-20px 
              h-20px 
              rounded-full 
              right--25px 
              top-50% 
              translate-y-[-50%] 
              flex 
              justify-center 
              items-center 
              text-12px 
              cursor-pointer 
              bg-#eef0f3 
              color-#9aa5b8 
              hover:bg-#067bef 
              hover:color-white`}
            onClick={handleAddBranch}
          >
            <PlusOutlined />
          </div>
        )}
        {type !== TopicType.main && children.length && (
          <div
            className="absolute right--30px top-0 w-30px h-full"
            onMouseOver={() => !collapsed && setShowCollapsePoint(true)}
            onMouseOut={() => !collapsed && setShowCollapsePoint(false)}
          />
        )}
        {/* 折叠按钮 */}
        {type !== TopicType.main && children?.length && showCollapsePoint && (
          <div
            className={`
              absolute
              rounded-full
              bg-white
              top-50%
              translate-y-[-50%]
              cursor-pointer
              hover:bg-#e6e6e6
              text-12px
              flex
              items-center
              justify-center
              ${collapsed ? "w-16px h-16px right--20px" : "w-10px h-10px right--15px"}
             `}
            onClick={handleToggleCollapse}
            style={{
              border: `1px solid ${fill.color1}`,
              color: fill.color1,
            }}
          >
            {collapsed && childrenCount}
          </div>
        )}
      </div>
    </>
  );
};

// 主题节点
register({
  shape: "mind-map-topic",
  width: 206,
  height: 70,
  effect: ["data"],
  component: component,
});

const baseNode = {
  shape: "mind-map-topic",
  data: {
    label: "",
    ...topicData,
  },
};

export default baseNode;
