import { TopicType } from "@/enum";
import { addTopic, getMindMapProjectByLocal } from "@/pages/mindmap/mindMap";
import { MindMapProjectInfo, TopicItem } from "@/types";
import { Cell, Graph, Node } from "@antv/x6";
import { message } from "antd";
import { cloneDeep } from "lodash-es";
import { uuid } from ".";

export const selectTopic = (graph: Graph, topic?: TopicItem) => {
  if (topic?.id) {
    setTimeout(() => {
      graph.resetSelection(topic.id);
      const node = graph.getCellById(topic?.id);
      node?.isNode() && graph.createTransformWidget(node);
    }, 100);
  }
};

/**
 * 添加同级主题
 * @param node
 * @param graph
 */
export const addPeerTopic = (
  node: Cell,
  graph: Graph,
  setMindProjectInfo?: (info: MindMapProjectInfo) => void
) => {
  if (!setMindProjectInfo) return;

  const parentNode =
    node.data.type === TopicType.main || !node.data?.parentId
      ? node
      : graph.getCellById(node.data.parentId);
  const type =
    node.data.type === TopicType.main ? TopicType.branch : node.data.type;
  if (parentNode?.isNode()) {
    const topic = addTopic(type, setMindProjectInfo, parentNode);

    selectTopic(graph, topic);
  }
};

/**
 * 添加子主题
 * @param node
 * @param setMindProjectInfo
 */
export const addChildrenTopic = (
  node: Node,
  setMindProjectInfo?: (info: MindMapProjectInfo) => void,
  graph?: Graph
) => {
  if (!setMindProjectInfo) return;

  const type =
    node.data?.type === TopicType.main ? TopicType.branch : TopicType.sub;
  const topic = addTopic(type, setMindProjectInfo, node);
  graph && selectTopic(graph, topic);
};

/**
 * 删除子主题
 * @param ids
 * @param setMindProjectInfo
 */
export const deleteTopics = (
  ids: string[],
  setMindProjectInfo?: (info: MindMapProjectInfo) => void
) => {
  const mindProjectInfo = getMindMapProjectByLocal();
  if (!mindProjectInfo || !setMindProjectInfo) return;
  const topics = cloneDeep(mindProjectInfo.topics);
  const filterTopics = (list: TopicItem[]): TopicItem[] => {
    const result: TopicItem[] = [];
    for (const item of list) {
      if (!ids.includes(item.id) || item.type === TopicType.main) {
        if (item.children?.length) {
          item.children = filterTopics(item.children);
        }
        result.push(item);
      }
    }
    return result;
  };

  mindProjectInfo.topics = filterTopics(topics);

  setMindProjectInfo(mindProjectInfo); // TODO 这个方法删除更新有问题
  localStorage.setItem("minMapProjectInfo", JSON.stringify(mindProjectInfo));
};

/**
 * 执行粘贴
 * @param graph
 * @param setMindProjectInfo
 */
export const handleMindmapPaste = (
  graph: Graph,
  setMindProjectInfo: (info: MindMapProjectInfo) => void
) => {
  // 读取剪切板数据
  navigator.clipboard.read().then((items) => {
    console.log("剪切板内容：", items);
    const currentNode = graph.getSelectedCells().find((cell) => cell.isNode());
    if (!currentNode) {
      message.warning("请先选择一个主题");
      return;
    }
    const item = items?.[0];
    if (item) {
      /**读取图片数据 */
      if (item.types[0] === "image/png") {
        item.getType("image/png").then((blob) => {
          const reader = new FileReader();
          reader.readAsDataURL(blob);
          reader.onload = function (event) {
            const dataUrl = event.target?.result as string;
            // 获取图片大小
            const img = new Image();
            img.src = dataUrl;
            img.onload = function () {
              const width = img.width;
              const height = img.height;
              // 插入图片
              currentNode.setData({
                extraModules: {
                  type: "image",
                  data: {
                    imageUrl: dataUrl,
                    width,
                    height,
                  },
                },
              });
            };
          };
        });
      }
      /**读取文本数据 */
      if (item.types[0] === "text/plain") {
        item.getType("text/plain").then((blob) => {
          const reader = new FileReader();
          reader.readAsText(blob);
          reader.onload = function (event) {
            const text = event.target?.result as string;
            // 内部复制方法
            if (text === "  ") {
              const nodes = localStorage.getItem("mindmap-copy-data");
              if (nodes) {
                JSON.parse(nodes)?.forEach((node: Node) => {
                  const data = node.data;
                  // 修改新的数据嵌套
                  data.id = uuid();
                  data.parentId = currentNode.id;
                  if (data.children?.length) {
                    data.children = traverseCopyData(data.children, data.id);
                  }

                  addTopic(
                    currentNode.data?.type === TopicType.main
                      ? TopicType.branch
                      : TopicType.sub,
                    setMindProjectInfo,
                    currentNode,
                    { ...data }
                  );
                });
              }
            } else {
              const topic = addTopic(
                currentNode.data?.type === TopicType.main
                  ? TopicType.branch
                  : TopicType.sub,
                setMindProjectInfo,
                currentNode,
                { label: text }
              );
              selectTopic(graph, topic);
            }
          };
        });
      }
    }
  });
};

const traverseCopyData = (list: TopicItem[], parentId: string): TopicItem[] => {
  return list.map((item) => {
    item.id = uuid();
    item.parentId = parentId;
    if (item.children?.length) {
      item.children = traverseCopyData(item.children, item.id);
    }
    return item;
  });
};